﻿using MySql.Data.MySqlClient;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Security.Cryptography;
using System.Text;


namespace DatabazeParkourovychSpotu
{
    public class MySqlDatabaze
    {
        #region Proměnné, vlastnosti, konstanty

        private MySqlConnection _pripojeni;
        private MySqlCommand _prikaz;
        private MySqlDataReader _dataReader;
        public static List<Spot> Spoty { get; private set; } = new List<Spot>();
        public static List<Obrazek> Obrazky { get; private set; } = new List<Obrazek>();
        public static List<Uzivatel> Uzivatele { get; private set; } = new List<Uzivatel>();
        public static List<Oblibene> Oblibene { get; private set; } = new List<Oblibene>();
        public static List<Hodnoceni> Hodnoceni { get; private set; } = new List<Hodnoceni>();

        private const string Server = "sql.endora.cz";
        private const string Port = "3309";
        private const string Database = "dpsc";
        private const string Uid = "dpsclocaluser";
        private const string Pwd = "kW5A35X_d5DS71aA";

        #endregion

        #region Inicializace

        // při zavolání prázdného konstruktoru se zavolá druhý konstruktor s paramtery, použijí se konstantní proměnné
        public MySqlDatabaze() : this(Server, Port, Database, Uid, Pwd) { }

        // konstruktor, zavolá metodu k vytvoření připojení a uloží ji do proměnné _pripojeni, poté volá metodu pro naplňění listů
        public MySqlDatabaze(string server, string port, string database, string uid, string pwd)
        {
            _pripojeni = VytvorPripojeni(server, port, database, uid, pwd);
            NacteniListu();
        }

        // vrátí do konstruktoru do proměnné _pripojeni MySqlConnection připojení
        private MySqlConnection VytvorPripojeni(string server, string port, string database, string uid, string pwd)
        {
            string pripojeniString = "Server=" + server + ";Port=" + port + ";Database=" + database + "; Uid=" + uid + ";Pwd=" + pwd + ";charset=utf8; convert zero datetime=True";
            return new MySqlConnection(pripojeniString);
        }

        #endregion

        #region Načtení listů

        // Volají se všechny metody pro načtení záznamů do listů
        private void NacteniListu()
        {
            NacteniSpotu();
            NacteniObrazku();
            NacteniHodnoceni();
            NacteniOblibene();
            NacteniUzivatelu();
        }

        // Načítá spoty do listu
        private void SpotyDoListu()
        {
            int id;
            string url, nazev, popis, castPrahy, adresa, metro, bus, tram, mapa, streetview, panorama, velikost, povrch, typ, obtiznost, obrazek, zverejneni, typSpotu;
            float hodnoceni;
            using (_prikaz = new MySqlCommand("SELECT s.id AS id, url, nazev, popis, praha, adresa, metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, obtiznost, typ_treninku, s.zverejneni, TRUNCATE(COALESCE(AVG(h.hodnoceni), 0), 2) as 'hodnoceni', MIN(nahled) as 'obrazek' FROM spoty s LEFT OUTER JOIN hodnoceni h ON s.id = h.id_spotu LEFT OUTER JOIN obrazky o ON s.id = o.id_spotu GROUP BY id, url, nazev, popis, praha, adresa, metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, typ_treninku, obtiznost, s.zverejneni ORDER BY nazev ASC", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Spoty.Count > 0) Spoty.Clear();
                    while (_dataReader.Read())
                    {
                        id = Convert.ToInt32(_dataReader["id"]);
                        url = _dataReader["url"].ToString();
                        nazev = _dataReader["nazev"].ToString();
                        popis = _dataReader["popis"].ToString();
                        castPrahy = _dataReader["praha"].ToString();
                        adresa = _dataReader["adresa"].ToString();
                        metro = _dataReader["metro"].ToString();
                        bus = _dataReader["bus"].ToString();
                        tram = _dataReader["tramvaj"].ToString();
                        mapa = _dataReader["mapa"].ToString();
                        streetview = _dataReader["streetview"].ToString();
                        panorama = _dataReader["panorama"].ToString();
                        velikost = _dataReader["velikost"].ToString();
                        povrch = _dataReader["povrch"].ToString();
                        typ = _dataReader["typ_treninku"].ToString();
                        obtiznost = _dataReader["obtiznost"].ToString();
                        hodnoceni = (float)Convert.ToDecimal(_dataReader["hodnoceni"]);
                        obrazek = _dataReader["obrazek"].ToString();
                        zverejneni = _dataReader["zverejneni"].ToString();
                        typSpotu = "veřejný";
                        Spoty.Add(new Spot(id, url, nazev, popis, castPrahy, adresa, metro, bus, tram, mapa, streetview, panorama, velikost, povrch, typ, obtiznost, hodnoceni, obrazek, zverejneni, typSpotu, -1));
                    }
                }
            }
        }

        // Načítá obrázky do listu
        private void ObrazkyDoListu()
        {
            using (_prikaz = new MySqlCommand("SELECT id, id_spotu, nahled, obrazek FROM obrazky ORDER BY nahled ASC", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Obrazky.Count > 0) Obrazky.Clear();
                    while (_dataReader.Read()) Obrazky.Add(new Obrazek(Convert.ToInt32(_dataReader["id"]), Convert.ToInt32(_dataReader["id_spotu"]), _dataReader["nahled"].ToString(), _dataReader["obrazek"].ToString(), "veřejný", -1));
                }
            }
        }

        // Načítá hodnocení do listu
        private void HodnoceniDoListu()
        {
            using (_prikaz = new MySqlCommand("SELECT id, id_spotu, hodnoceni, id_uzivatele, zverejneni FROM hodnoceni", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Hodnoceni.Count > 0) Hodnoceni.Clear();
                    while (_dataReader.Read())
                        Hodnoceni.Add(new Hodnoceni(Convert.ToInt32(_dataReader["id"]), Convert.ToInt32(_dataReader["id_spotu"]), Convert.ToInt32(_dataReader["hodnoceni"]), Convert.ToInt32(_dataReader["id_uzivatele"]), _dataReader["zverejneni"].ToString(), 1, 1));
                }
            }
        }

        // Načítá oblíbené do listu
        private void OblibeneDoListu()
        {
            using (_prikaz = new MySqlCommand("SELECT id, id_spotu, id_uzivatele, zverejneni FROM oblibene", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Oblibene.Count > 0) Oblibene.Clear();
                    while (_dataReader.Read())
                        Oblibene.Add(new Oblibene(Convert.ToInt32(_dataReader["id"]), Convert.ToInt32(_dataReader["id_spotu"]), Convert.ToInt32(_dataReader["id_uzivatele"]), _dataReader["zverejneni"].ToString(), 1, 1));
                }
            }
        }

        // Načítá uživatele do listu
        private void UzivateleDoListu()
        {
            using (_prikaz = new MySqlCommand("SELECT id, url, uzivatel, email, heslo, obrazek, datum, aktivace, stav FROM uzivatele", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Uzivatele.Count > 0) Uzivatele.Clear();
                    while (_dataReader.Read())
                        Uzivatele.Add(new Uzivatel(Convert.ToInt32(_dataReader["id"]), _dataReader["url"].ToString(), _dataReader["uzivatel"].ToString(), _dataReader["email"].ToString(), _dataReader["heslo"].ToString(), _dataReader["obrazek"].ToString(), _dataReader["datum"].ToString(), _dataReader["aktivace"].ToString(), Convert.ToInt32(_dataReader["stav"]), 1, 1));
                }
            }
        }

        // Metody NacteniXXX (XXX = záznam) volají metody pro načtení záznamů do listu
        // Podle stavu připojení buď otevřou nové připjení nebo použijí současné připojení
        public void NacteniSpotu()
        {
            if (_pripojeni.State == ConnectionState.Open)
                SpotyDoListu();
            else
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    SpotyDoListu();
                }
        }
        public void NacteniObrazku()
        {
            if (_pripojeni.State == ConnectionState.Open)
                ObrazkyDoListu();
            else
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    ObrazkyDoListu();
                }
        }
        public void NacteniHodnoceni()
        {
            if (_pripojeni.State == ConnectionState.Open)
                HodnoceniDoListu();
            else
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    HodnoceniDoListu();
                }
        }
        public void NacteniOblibene()
        {
            if (_pripojeni.State == ConnectionState.Open)
                OblibeneDoListu();
            else
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    OblibeneDoListu();
                }
        }
        public void NacteniUzivatelu()
        {
            if (_pripojeni.State == ConnectionState.Open)
                UzivateleDoListu();
            else
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    UzivateleDoListu();
                }
        }
        #endregion

        #region Synchronizace dat

        // SynchronizaceDat - Volají se metody pro synchronizaci záznamů v MySql a SqlCe databázích
        public void SynchronizaceDat(SqlCeDatabaze sqlCe)
        {
            using (_pripojeni)
            {
                _pripojeni.Open();
                SqlCeUpdateSpoty(sqlCe);
                SqlCeUpdateObrazky(sqlCe);
                SqlCeToMySqlHodnoceni(sqlCe);
                SqlCeToMySqlOblibene(sqlCe);
                SqlCeToMySqlUzivatele(sqlCe);
            }
        }

        // slouží pro aktualizaci spotů v SqlCe databázi
        private void SqlCeUpdateSpoty(SqlCeDatabaze sqlCe)
        {
            bool spotyFetch = false, fetch = false;
            int id = -1;

            // vyberu nestejné záznamy obou listů a použiji záznam (spot) z SqlCe databáze
            List<Spot> noveSpoty = SqlCeDatabaze.Spoty.Where(x => !Spoty.Any(x1 => x1.Nazev == x.Nazev && x1.Popis == x.Popis && x1.Praha == x.Praha && x1.Adresa == x.Adresa && x1.Metro == x.Metro && x1.Bus == x.Bus && x1.Tramvaj == x.Tramvaj && x1.Mapa == x.Mapa && x1.Streetview == x.Streetview && x1.Panorama == x.Panorama && x1.Velikost == x.Velikost && x1.Obtiznost == x.Obtiznost && x1.Povrch == x.Povrch && x1.Typ == x.Typ)).ToList();

            // procházím nově vytvořený list s nestejnými záznamy
            foreach (Spot spot in noveSpoty)
            {
                spotyFetch = true;
                fetch = false;

                // pokud spoty v obou databázích mají stejné ID, ale v sqlCe databázi je spot typu soukromý, tak se spot přidá do MySql databáze, zjistí se jeho ID, smaže se z mySql databáze a upraví se ID v sqlCe databázi
                foreach (Spot s in Spoty.Where(s => s.ID == spot.ID))
                    if (spot.TypSpotu == "soukromý")
                    {
                        fetch = true;
                        using (_prikaz = new MySqlCommand("INSERT INTO spoty(id, url, nazev, popis, praha, adresa, metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, obtiznost, typ_treninku, zverejneni) VALUES(@id, @url, @nazev, @popis, @praha, @adresa, @metro, @bus, @tramvaj, @mapa, @streetview, @panorama, @velikost, @povrch, @obtiznost, @typTreninku, @zverejneni)", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@id", spot.ID);
                            _prikaz.Parameters.AddWithValue("@url", spot.Url);
                            _prikaz.Parameters.AddWithValue("@nazev", spot.Nazev);
                            _prikaz.Parameters.AddWithValue("@popis", spot.Popis);
                            _prikaz.Parameters.AddWithValue("@praha", spot.Praha);
                            _prikaz.Parameters.AddWithValue("@adresa", spot.Adresa);
                            _prikaz.Parameters.AddWithValue("@metro", spot.Metro);
                            _prikaz.Parameters.AddWithValue("@bus", spot.Bus);
                            _prikaz.Parameters.AddWithValue("@tramvaj", spot.Tramvaj);
                            _prikaz.Parameters.AddWithValue("@mapa", spot.Mapa);
                            _prikaz.Parameters.AddWithValue("@streetview", spot.Streetview);
                            _prikaz.Parameters.AddWithValue("@panorama", spot.Panorama);
                            _prikaz.Parameters.AddWithValue("@velikost", spot.Velikost);
                            _prikaz.Parameters.AddWithValue("@povrch", spot.Povrch);
                            _prikaz.Parameters.AddWithValue("@obtiznost", spot.Obtiznost);
                            _prikaz.Parameters.AddWithValue("@typTreninku", spot.Typ);
                            _prikaz.Parameters.AddWithValue("@zverejneni", spot.Zverejneni);
                            _prikaz.ExecuteNonQuery();
                        }
                        using (_prikaz = new MySqlCommand("SELECT id FROM spoty WHERE nazev = @nazev AND popis = @popis AND adresa = @adresa AND mapa = @mapa", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@nazev", spot.Nazev);
                            _prikaz.Parameters.AddWithValue("@popis", spot.Popis);
                            _prikaz.Parameters.AddWithValue("@adresa", spot.Adresa);
                            _prikaz.Parameters.AddWithValue("@mapa", spot.Mapa);
                            using (_dataReader = _prikaz.ExecuteReader())
                                while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                        }
                        using (_prikaz = new MySqlCommand("DELETE FROM spoty WHERE nazev = @nazev AND popis = @popis AND adresa = @adresa AND mapa = @mapa", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@nazev", spot.Nazev);
                            _prikaz.Parameters.AddWithValue("@popis", spot.Popis);
                            _prikaz.Parameters.AddWithValue("@adresa", spot.Adresa);
                            _prikaz.Parameters.AddWithValue("@mapa", spot.Mapa);
                            _prikaz.ExecuteNonQuery();
                        }
                        sqlCe.UpdateSpoty(id, spot.Url, spot.Nazev, spot.Popis, spot.Praha, spot.Adresa, spot.Metro, spot.Bus, spot.Tramvaj, spot.Mapa, spot.Streetview, spot.Panorama, spot.Velikost, spot.Povrch, spot.Obtiznost, spot.Typ, DateTime.Parse(spot.Zverejneni), spot.TypSpotu, spot.IdUzivatele, "ID");
                    }
                    // když je spot v SqlCe i MySql databázi a mají stejné ID a spot v SqlCe je veřejný, ale v MySql databázi je změněný (např. upravená doprava),
                    // tak se aktualizuje tento spot v SqlCe databázi, aby oba spoty v obou databázích byly stejné
                    else if (spot.TypSpotu == "veřejný" && !(s.Url == spot.Url && s.Nazev == spot.Nazev && s.Popis == spot.Popis && s.Praha == spot.Praha && s.Adresa == spot.Adresa && s.Metro == spot.Metro && s.Bus == spot.Bus && s.Tramvaj == spot.Tramvaj && s.Mapa == spot.Mapa && s.Streetview == spot.Streetview && s.Panorama == spot.Panorama && s.Velikost == spot.Velikost && s.Povrch == spot.Povrch && s.Obtiznost == spot.Obtiznost && s.Typ == spot.Typ && s.Zverejneni == spot.Zverejneni))
                    {
                        fetch = true;
                        sqlCe.UpdateSpoty(s.ID, s.Url, s.Nazev, s.Popis, s.Praha, s.Adresa, s.Metro, s.Bus, s.Tramvaj, s.Mapa, s.Streetview, s.Panorama, s.Velikost, s.Povrch, s.Obtiznost, s.Typ, DateTime.Parse(s.Zverejneni), s.TypSpotu, -1, "ID");
                    }

                // když nemají spoty stejné ID a jedná se veřejný spot, tak ho smažu
                if (!fetch && spot.TypSpotu == "veřejný") sqlCe.DeleteZaznam(spot.ID, "spoty");
            }
            if (spotyFetch) NacteniSpotu();
        }

        // slouží pro update obrázků v SqlCe databázi
        private void SqlCeUpdateObrazky(SqlCeDatabaze sqlCe)
        {
            bool obrazkyFetch = false, fetch = false;
            int id = -1;

            // vyberu nestejné záznamy obou listů a použiji záznam (obrázek) z SqlCe databáze
            List<Obrazek> noveObrazky = SqlCeDatabaze.Obrazky.Where(x => !Obrazky.Any(x1 => x1.Nahled == x.Nahled && x1.Fotka == x.Fotka)).ToList();

            // procházím nově vytvořený list s nestejnými záznamy
            foreach (Obrazek obrazek in noveObrazky)
            {
                obrazkyFetch = true;
                fetch = false;

                // pokud obrázky v obou databázích mají stejné ID, ale v sqlCe databázi je obrázek typu soukromý, tak se obrázek přidá do MySql databáze, zjistí se jeho ID, smaže se z mySql databáze a upraví se ID v sqlCe databázi
                foreach (Obrazek o in Obrazky.Where(o => o.ID == obrazek.ID))
                    if (obrazek.TypObrazku == "soukromý")
                    {
                        fetch = true;
                        using (_prikaz = new MySqlCommand("INSERT INTO obrazky(id, id_spotu, nahled, obrazek) VALUES(@id, @idSpotu, @nahled, @obrazek)", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@id", obrazek.ID);
                            _prikaz.Parameters.AddWithValue("@idSpotu", obrazek.IdSpotu);
                            _prikaz.Parameters.AddWithValue("@nahled", obrazek.Nahled);
                            _prikaz.Parameters.AddWithValue("@obrazek", obrazek.Fotka);
                            _prikaz.ExecuteNonQuery();
                        }
                        using (_prikaz = new MySqlCommand("SELECT id FROM obrazky WHERE id_spotu = @idSpotu AND nahled = @nahled", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@idSpotu", obrazek.IdSpotu);
                            _prikaz.Parameters.AddWithValue("@nahled", obrazek.Nahled);
                            using (_dataReader = _prikaz.ExecuteReader())
                                while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                        }
                        using (_prikaz = new MySqlCommand("DELETE FROM obrazky WHERE id_spotu = @idSpotu AND nahled = @nahled", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@idSpotu", obrazek.IdSpotu);
                            _prikaz.Parameters.AddWithValue("@nahled", obrazek.Nahled);
                            _prikaz.ExecuteNonQuery();
                        }
                        sqlCe.UpdateObrazky(id, obrazek.IdSpotu, obrazek.Nahled, obrazek.Fotka, obrazek.TypObrazku, obrazek.IdUzivatele);
                    }
                    // když je obrázek v SqlCe i MySql databázi a mají stejné ID a obrázek v SqlCe je veřejný, ale v MySql databázi je změněný,
                    // tak se aktualizuje tento obrázek v SqlCe databázi, aby oba obrázky v obou databázích byly stejné
                    else if (obrazek.TypObrazku == "veřejný" && !(o.IdSpotu == obrazek.IdSpotu && o.Nahled == obrazek.Nahled && o.Fotka == obrazek.Fotka))
                    {
                        fetch = true;
                        sqlCe.UpdateObrazky(o.ID, o.IdSpotu, o.Nahled, o.Fotka, o.TypObrazku, -1);
                    }

                // když nemají obrázky stejné ID a jedná se veřejný obrázek, tak ho smažu
                if (!fetch && obrazek.TypObrazku == "veřejný") sqlCe.DeleteZaznam(obrazek.ID, "obrazky");
            }
            if (obrazkyFetch) NacteniSpotu();
        }

        // Metoda SqlCeToMySqlHodnoceni - slouží pro překopírování hodnocení z SqlCe do MySql
        // překopírování nového hodnocení z MySql do SqlCe databáze
        // překopírování nového hodnocení z SqlCe do MySql databáze
        // Při smazání hodnocení z MySql se smaže také v SqlCe
        // když je hodnocení v obou databázích, ale má změněný údaj v MySql, tak se hodnocení v SqlCe upraví podle MySql hodnocení
        // když má hodnocení z SqlCe stejné ID jako hodnocení z MySql, ale jsou to dvě různé hodnocení, tak se změní ID a obě dvě se vytvoří v protější databázi
        private void SqlCeToMySqlHodnoceni(SqlCeDatabaze sqlCe)
        {
            int id = 0;
            bool hodnoceniFetch = false, fetch = false;

            // vybírám hodnocení, které nejsou v MySql, ale jsou v SlqCe databázi a také se berou pouze veřejné hodnocení a také, které jsou v obou, ale liší se v údajích
            List<Hodnoceni> noveHodnoceni = SqlCeDatabaze.Hodnoceni.Where(x => !Hodnoceni.Any(x1 => x1.IdSpotu == x.IdSpotu && x1.IdUzivatele == x.IdUzivatele && x1.HodnoceniSpotu == x.HodnoceniSpotu) && (SqlCeDatabaze.Spoty.Any(x2 => x2.TypSpotu == "veřejný" && x2.ID == x.IdSpotu))).Union(SqlCeDatabaze.Hodnoceni.Where(x => Hodnoceni.Any(x1 => (x1.IdSpotu == x.IdSpotu && x1.IdUzivatele == x.IdUzivatele && x1.HodnoceniSpotu == x.HodnoceniSpotu && x.SqlCe == 1 && x.MySql == 0))).Union(SqlCeDatabaze.Hodnoceni.Where(x => SqlCeDatabaze.Spoty.Any(x1 => x1.ID == x.IdSpotu && x1.TypSpotu == "soukromý")))).ToList();
            foreach (Hodnoceni hodnoceni in noveHodnoceni)
            {
                hodnoceniFetch = true;
                fetch = false;

                // když mají hodnocení v obou listech stejné ID
                foreach (Hodnoceni h in Hodnoceni.Where(h => h.ID == hodnoceni.ID))
                    // a mají něco změněno a nejsou to jiné záznamy, tak srovnám tyto 2 záznamy podle mySql databáze
                    if (!(h.IdSpotu == hodnoceni.IdSpotu && h.HodnoceniSpotu == hodnoceni.HodnoceniSpotu && h.IdUzivatele == hodnoceni.IdUzivatele && h.Zverejneni == hodnoceni.Zverejneni) && !((h.IdSpotu != hodnoceni.IdSpotu && h.IdUzivatele != hodnoceni.IdUzivatele) || (h.IdSpotu != hodnoceni.IdSpotu && h.IdUzivatele == hodnoceni.IdUzivatele) || (h.IdSpotu == hodnoceni.IdSpotu && h.IdUzivatele != hodnoceni.IdUzivatele)))
                    {
                        fetch = true;
                        sqlCe.UpdateHodnoceni(h.ID, h.IdSpotu, h.HodnoceniSpotu, h.IdUzivatele, DateTime.Parse(h.Zverejneni), hodnoceni.SqlCe, hodnoceni.MySql);
                    }

                // když se hodnocení liší v ID, ale mají stejné id spotu a stejného uživatele, tak se smaže hodnocení z SqlCe
                foreach (Hodnoceni h in Hodnoceni.Where(h => h.IdSpotu == hodnoceni.IdSpotu && h.IdUzivatele == hodnoceni.IdUzivatele && h.ID != hodnoceni.ID))
                {
                    fetch = true;
                    sqlCe.DeleteZaznam(hodnoceni.ID, "hodnoceni");
                    break;
                }

                // pokud není ID stejné, a nebo je stejné, ale jedná se o jiné hodnocení
                if (!fetch)
                {
                    // A) pokud není ID stejné a pokud se v SqlCe jeví, že hodnocení již bylo v MySql, ale v MySql už není, tak smažu hodnocení z SqlCe
                    if (hodnoceni.SqlCe == 1 && hodnoceni.MySql == 1)
                    {
                        sqlCe.DeleteZaznam(hodnoceni.ID, "hodnoceni");
                    }
                    // B) pokud není ID stejné, a nebo je stejné, ale jedná se o jiné hodnocení
                    // a pokud se v SqlCe jeví, že v MySql ještě není, a ani tam dříve nebylo, tak přidám hodnocení do MySql a v SqlCe mu změním ID
                    else
                    {
                        using (_prikaz = new MySqlCommand("INSERT INTO hodnoceni(id_spotu, hodnoceni, id_uzivatele, zverejneni) VALUES(@idSpotu, @hodnoceni, @idUzivatele, @zverejneni)", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@idSpotu", hodnoceni.IdSpotu);
                            _prikaz.Parameters.AddWithValue("@hodnoceni", hodnoceni.HodnoceniSpotu);
                            _prikaz.Parameters.AddWithValue("@idUzivatele", hodnoceni.IdUzivatele);
                            _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Parse(hodnoceni.Zverejneni));
                            _prikaz.ExecuteNonQuery();
                        }                    
                        using (_prikaz = new MySqlCommand("SELECT id FROM hodnoceni WHERE id_spotu = @idSpotu AND hodnoceni = @hodnoceni AND id_uzivatele = @idUzivatele AND zverejneni = @zverejneni", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@idSpotu", hodnoceni.IdSpotu);
                            _prikaz.Parameters.AddWithValue("@hodnoceni", hodnoceni.HodnoceniSpotu);
                            _prikaz.Parameters.AddWithValue("@idUzivatele", hodnoceni.IdUzivatele);
                            _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Parse(hodnoceni.Zverejneni));
                            using (_dataReader = _prikaz.ExecuteReader())
                                while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                        }

                        // získá se ID z sqlCe databáze a pokud je větší než ID z mySql, tak se upraví záznam v MySql, poté se upraví záznam v SqlCe
                        int sqlCeId = sqlCe.ZiskaniId("hodnoceni");

                        if (sqlCeId > id)
                        {
                            using (_prikaz = new MySqlCommand("UPDATE hodnoceni SET id = @id WHERE id_uzivatele = @idUzivatele && id_spotu = @idSpotu", _pripojeni))
                            {
                                _prikaz.Parameters.AddWithValue("@id", sqlCeId);
                                _prikaz.Parameters.AddWithValue("@idSpotu", hodnoceni.IdSpotu);
                                _prikaz.Parameters.AddWithValue("@idUzivatele", hodnoceni.IdUzivatele);
                                _prikaz.ExecuteNonQuery();
                            }
                        }
                        id = (id >= sqlCeId) ? id : sqlCeId;
                        sqlCe.UpdateHodnoceni(id, hodnoceni.IdSpotu, hodnoceni.HodnoceniSpotu, hodnoceni.IdUzivatele, DateTime.Parse(hodnoceni.Zverejneni), 1, 1);
                    }
                }
            }
            if (hodnoceniFetch) NacteniHodnoceni();
        }

        // Metoda SqlCeToMySqlOblibene - slouží pro překopírování oblíbených spotů z SqlCe do MySql
        // překopírování nového oblíbeného z MySql do SqlCe databáze
        // překopírování nového oblíbeného z SqlCe do MySql databáze
        // Při smazání oblíbeného z MySql se smaže také v SqlCe
        // když je oblíbené v obou databázích, ale má změněný údaj v MySql, tak se oblíbené v SqlCe upraví podle MySql
        // když má oblíbené z SqlCe stejné ID jako oblíbené z MySql, ale jsou to dvě různé oblíbené, tak se změní ID a obě dvě se vytvoří v protější databázi
        private void SqlCeToMySqlOblibene(SqlCeDatabaze sqlCe)
        {
            int id = 0;
            bool oblibeneFetch = false, fetch = false;

            // vybírám oblíbené, které nejsou v MySql, ale jsou v SqlCe databázi a také se berou pouze veřejné oblíbené a také, které jsou v obou, ale liší se v údajích
            List<Oblibene> noveOblibene = SqlCeDatabaze.Oblibene.Where(x => !Oblibene.Any(x1 => x1.IdSpotu == x.IdSpotu && x1.IdUzivatele == x.IdUzivatele) && (SqlCeDatabaze.Spoty.Any(x2 => x2.TypSpotu == "veřejný" && x2.ID == x.IdSpotu))).Union(SqlCeDatabaze.Oblibene.Where(x => Oblibene.Any(x1 => (x1.IdSpotu == x.IdSpotu && x1.IdUzivatele == x.IdUzivatele && x.SqlCe == 1 && x.MySql == 0)))).Union(SqlCeDatabaze.Oblibene.Where(x => Oblibene.Any(x1 => (x1.IdSpotu == x.IdSpotu && x1.IdUzivatele == x.IdUzivatele && x.SqlCe == 0 && x.MySql == 1)))).ToList();
            foreach (Oblibene oblibene in noveOblibene)
            {
                oblibeneFetch = true;
                fetch = false;

                // když mají oblíbené v obou listech stejné ID
                foreach (Oblibene o in Oblibene.Where(o => o.ID == oblibene.ID))
                    // a mají něco změněno a nejsou to jiné záznamy, tak srovnám tyto 2 záznamy podle mySql databáze
                    if (!(o.IdSpotu == oblibene.IdSpotu && o.IdUzivatele == oblibene.IdUzivatele && o.Zverejneni == oblibene.Zverejneni) && !((o.IdSpotu != oblibene.IdSpotu && o.IdUzivatele != oblibene.IdUzivatele) || (o.IdSpotu != oblibene.IdSpotu && o.IdUzivatele == oblibene.IdUzivatele) || (o.IdSpotu == oblibene.IdSpotu && o.IdUzivatele != oblibene.IdUzivatele)))
                    {
                        fetch = true;
                        sqlCe.UpdateOblibene(o.ID, o.IdSpotu, o.IdUzivatele, DateTime.Parse(o.Zverejneni), oblibene.SqlCe, oblibene.MySql);
                    }

                // když se oblíbené liší v ID, ale mají stejné id spotu a stejného uživatele, tak se smaže oblíbené z SqlCe
                foreach (Oblibene o in Oblibene.Where(o => o.IdSpotu == oblibene.IdSpotu && o.IdUzivatele == oblibene.IdUzivatele && o.ID != oblibene.ID))
                {
                    fetch = true;
                    sqlCe.DeleteZaznam(oblibene.ID, "oblibene");
                    break;
                }

                // pokud není ID stejné, a nebo je stejné, ale jedná se o jiné oblíbené
                if (!fetch)
                {

                    // A) pokud není ID stejné a pokud se v SqlCe jeví, že oblíbené již bylo v MySql, ale v MySql už není, tak smažu oblíbené z SqlCe
                    if (oblibene.SqlCe == 1 && oblibene.MySql == 1)
                    {
                        sqlCe.DeleteZaznam(oblibene.ID, "oblibene");
                    }
                    // B) pokud se v SqlCe databázi jeví oblíbené jako smazané, tak smažu oblíbené z obou databází
                    else if (oblibene.SqlCe == 0 && oblibene.MySql == 1)
                    {
                        sqlCe.DeleteZaznam(oblibene.ID, "oblibene");
                        using (_prikaz = new MySqlCommand("DELETE FROM oblibene WHERE id_spotu = @idSpotu AND id_uzivatele = @idUzivatele", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@idSpotu", oblibene.IdSpotu);
                            _prikaz.Parameters.AddWithValue("@idUzivatele", oblibene.IdUzivatele);
                            _prikaz.ExecuteNonQuery();
                        }
                    }
                    // C) pokud není ID stejné, a nebo je stejné, ale jedná se o jiné oblíbené
                    // C) a pokud se v SqlCe jeví, že v MySql ještě není, a ani tam dříve nebylo, tak přidám oblíbené do MySql a v SqlCe mu změním ID
                    else
                    {
                        using (_prikaz = new MySqlCommand("INSERT INTO oblibene(id_spotu, id_uzivatele, zverejneni) VALUES(@idSpotu, @idUzivatele, @zverejneni)", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@idSpotu", oblibene.IdSpotu);
                            _prikaz.Parameters.AddWithValue("@idUzivatele", oblibene.IdUzivatele);
                            _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Parse(oblibene.Zverejneni));
                            _prikaz.ExecuteNonQuery();
                        }
                        using (_prikaz = new MySqlCommand("SELECT id FROM oblibene WHERE id_spotu = @idSpotu AND id_uzivatele = @idUzivatele AND zverejneni = @zverejneni", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@idSpotu", oblibene.IdSpotu);
                            _prikaz.Parameters.AddWithValue("@idUzivatele", oblibene.IdUzivatele);
                            _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Parse(oblibene.Zverejneni));
                            using (_dataReader = _prikaz.ExecuteReader())
                                while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                        }

                        // získá se ID z sqlCe databáze a pokud je větší než ID z mySql, tak se upraví záznam v MySql, poté se upraví záznam v SqlCe
                        int sqlCeId = sqlCe.ZiskaniId("oblibene");

                        if (sqlCeId > id)
                        {
                            using (_prikaz = new MySqlCommand("UPDATE oblibene SET id = @id WHERE id_uzivatele = @idUzivatele && id_spotu = @idSpotu", _pripojeni))
                            {
                                _prikaz.Parameters.AddWithValue("@id", sqlCeId);
                                _prikaz.Parameters.AddWithValue("@idSpotu", oblibene.IdSpotu);
                                _prikaz.Parameters.AddWithValue("@idUzivatele", oblibene.IdUzivatele);
                                _prikaz.ExecuteNonQuery();
                            }
                        }
                        id = (id >= sqlCeId) ? id : sqlCeId;
                        sqlCe.UpdateOblibene(id, oblibene.IdSpotu, oblibene.IdUzivatele, DateTime.Parse(oblibene.Zverejneni), 1, 1);
                    }
                }
            }
            if (oblibeneFetch) NacteniOblibene();
        }

        // Metoda SqlCeToMySqlUzivatele - slouží pro překopírování uživatelů z SqlCe do MySql
        // překopírování nového uživatele z MySql do SqlCe databáze
        // překopírování nového uživatele z SqlCe do MySql databáze
        // Při smazání uživatele z MySql se smaže také v SqlCe
        // když je uživatel v obou databázích, ale má změněný udaj v MySql, tak se uživatel v SqlCe upraví podle MySql uživatele
        // když má uživatel z SqlCe stejné ID jako uživatel z MySql, ale jsou to dva různí uživatelé, tak se změní ID a oba dva se vytvoří v protější databázi
        private void SqlCeToMySqlUzivatele(SqlCeDatabaze sqlCe)
        {
            int id = 0;
            bool uzivateleFetch = false, fetch = false;

            // vybírám uživatele, kteří nejsou v MySql, ale jsou v SqlCe databázi a také, kteří jsou v obou, ale liší se v údajích
            List<Uzivatel> novyUzivatele = SqlCeDatabaze.Uzivatele.Where(x => !Uzivatele.Any(x1 => x1.UzivatelskeJmeno == x.UzivatelskeJmeno && x1.Email == x.Email && x1.Heslo == x.Heslo && x1.Obrazek == x.Obrazek && x1.Stav == x.Stav)).Union(SqlCeDatabaze.Uzivatele.Where(x => Uzivatele.Any(x1 => x1.ID == x.ID && x1.UzivatelskeJmeno != x.UzivatelskeJmeno))).ToList();
            foreach (Uzivatel uzivatel in novyUzivatele)
            {
                uzivateleFetch = true;
                fetch = false;

                // když mají uživatelé v obou listech stejné ID
                foreach (Uzivatel u in Uzivatele.Where(u => u.ID == uzivatel.ID))
                    // a mají něco změněno a nejsou to jiné záznamy, tak srovnám tyto 2 záznamy podle mySql databáze
                    if (!(u.Url == uzivatel.Url && u.UzivatelskeJmeno == uzivatel.UzivatelskeJmeno && u.Email == uzivatel.Email && u.Heslo == uzivatel.Heslo && u.Obrazek == uzivatel.Obrazek && u.Datum == uzivatel.Datum && u.Aktivace == uzivatel.Aktivace && u.Stav == uzivatel.Stav) && !(u.UzivatelskeJmeno != uzivatel.UzivatelskeJmeno))
                    {
                        fetch = true;
                        sqlCe.UpdateUzivatele(u.ID, u.Url, u.UzivatelskeJmeno, u.Email, u.Heslo, u.Obrazek, DateTime.Parse(u.Datum), u.Aktivace, u.Stav, uzivatel.SqlCe, uzivatel.MySql);
                    }

                // když se uživatel liší v ID, ale je to stejný uživatel, tak se smaže uživatel z SqlCe
                foreach (Uzivatel u in Uzivatele.Where(u => u.UzivatelskeJmeno == uzivatel.UzivatelskeJmeno && u.ID != uzivatel.ID))
                {
                    fetch = true;
                    sqlCe.DeleteZaznam(uzivatel.ID, "uzivatele");
                    break;
                }

                // pokud není ID stejné, a nebo je stejné, ale jedná se o jiné uživatele
                if (!fetch)
                {
                    // A) pokud není ID stejné a pokud se v SqlCe jeví, že uživatel již byl v MySql, ale v MySql už není, tak smažu uživatele z SqlCe
                    if (uzivatel.SqlCe == 1 && uzivatel.MySql == 1)
                    {
                        sqlCe.DeleteZaznam(uzivatel.ID, "uzivatele");
                    }
                    // B) pokud není ID stejné, a nebo je stejné, ale jedná se o jiné uživatele
                    // B) a pokud se v SqlCe jeví, že v MySql ještě není, a ani tam dříve nebyl, tak přidám uživatele do MySql a v SqlCe mu změním ID
                    else
                    {
                        using (_prikaz = new MySqlCommand("INSERT INTO uzivatele(url, uzivatel, email, heslo, obrazek, datum, aktivace, stav) VALUES(@url, @uzivatel, @email, @heslo, @obrazek, @datum, @aktivace, @stav)", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@url", uzivatel.Url);
                            _prikaz.Parameters.AddWithValue("@uzivatel", uzivatel.UzivatelskeJmeno);
                            _prikaz.Parameters.AddWithValue("@email", uzivatel.Email);
                            _prikaz.Parameters.AddWithValue("@heslo", uzivatel.Heslo);
                            _prikaz.Parameters.AddWithValue("@obrazek", uzivatel.Obrazek);
                            _prikaz.Parameters.AddWithValue("@datum", DateTime.Parse(uzivatel.Datum));
                            _prikaz.Parameters.AddWithValue("@aktivace", uzivatel.Aktivace);
                            _prikaz.Parameters.AddWithValue("@stav", uzivatel.Stav);
                            _prikaz.ExecuteNonQuery();
                        }
                        using (_prikaz = new MySqlCommand("SELECT id FROM uzivatele WHERE uzivatel = @uzivatel AND email = @email AND heslo = @heslo AND datum = @datum", _pripojeni))
                        {
                            _prikaz.Parameters.AddWithValue("@uzivatel", uzivatel.UzivatelskeJmeno);
                            _prikaz.Parameters.AddWithValue("@email", uzivatel.Email);
                            _prikaz.Parameters.AddWithValue("@heslo", uzivatel.Heslo);
                            _prikaz.Parameters.AddWithValue("@datum", DateTime.Parse(uzivatel.Datum));
                            using (_dataReader = _prikaz.ExecuteReader())
                                while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                        }

                        // získá se ID z sqlCe databáze a pokud je větší než ID z mySql, tak se upraví záznam v MySql, poté se upraví záznam v SqlCe
                        int sqlCeId = sqlCe.ZiskaniId("uzivatele");

                        if (sqlCeId > id)
                        {
                            using (_prikaz = new MySqlCommand("UPDATE uzivatele SET id = @id WHERE uzivatel = @uzivatel", _pripojeni))
                            {
                                _prikaz.Parameters.AddWithValue("@id", sqlCeId);
                                _prikaz.Parameters.AddWithValue("@uzivatel", uzivatel.UzivatelskeJmeno);
                                _prikaz.ExecuteNonQuery();
                            }
                        }
                        id = (id >= sqlCeId) ? id : sqlCeId;
                        sqlCe.UpdateUzivatele(id, uzivatel.Url, uzivatel.UzivatelskeJmeno, uzivatel.Email, uzivatel.Heslo, uzivatel.Obrazek, DateTime.Parse(uzivatel.Datum), uzivatel.Aktivace, uzivatel.Stav, 1, 1);
                    }
                }

            }
            if (uzivateleFetch) NacteniUzivatelu();
        }

        #endregion

        #region Vytváření záznamů

        // vytvoří se nový spot v MySql databázi, uloží se jeho ID a smaže se a pomocí metody UpdateSpoty u SqlCe databáze se vytvoří i v SqlCe databázi
        public int VytvoreniSpotu(SqlCeDatabaze sqlCe, string url, string nazev, string popis, string praha, string adresa, string metro, string bus, string tramvaj, string mapa, string streetview, string panorama, string velikost, string povrch, string obtiznost, string typTreninku, DateTime zverejneni, string typSpotu, int idUzivatele)
        {
            int id = -1;
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new MySqlCommand("INSERT INTO spoty(url, nazev, popis, praha, adresa, metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, obtiznost, typ_treninku, zverejneni) VALUES(@url, @nazev, @popis, @praha, @adresa, @metro, @bus, @tramvaj, @mapa, @streetview, @panorama, @velikost, @povrch, @obtiznost, @typTreninku, @zverejneni)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@url", url);
                    _prikaz.Parameters.AddWithValue("@nazev", nazev);
                    _prikaz.Parameters.AddWithValue("@popis", popis);
                    _prikaz.Parameters.AddWithValue("@praha", praha);
                    _prikaz.Parameters.AddWithValue("@adresa", adresa);
                    _prikaz.Parameters.AddWithValue("@metro", metro);
                    _prikaz.Parameters.AddWithValue("@bus", bus);
                    _prikaz.Parameters.AddWithValue("@tramvaj", tramvaj);
                    _prikaz.Parameters.AddWithValue("@mapa", mapa);
                    _prikaz.Parameters.AddWithValue("@streetview", streetview);
                    _prikaz.Parameters.AddWithValue("@panorama", panorama);
                    _prikaz.Parameters.AddWithValue("@velikost", velikost);
                    _prikaz.Parameters.AddWithValue("@povrch", povrch);
                    _prikaz.Parameters.AddWithValue("@obtiznost", obtiznost);
                    _prikaz.Parameters.AddWithValue("@typTreninku", typTreninku);
                    _prikaz.Parameters.AddWithValue("@zverejneni", zverejneni);
                    _prikaz.ExecuteNonQuery();
                }

                using (_prikaz = new MySqlCommand("SELECT id FROM spoty WHERE nazev = @nazev AND popis = @popis AND adresa = @adresa AND mapa = @mapa", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@nazev", nazev);
                    _prikaz.Parameters.AddWithValue("@popis", popis);
                    _prikaz.Parameters.AddWithValue("@adresa", adresa);
                    _prikaz.Parameters.AddWithValue("@mapa", mapa);
                    using (_dataReader = _prikaz.ExecuteReader())
                        while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                }
                using (_prikaz = new MySqlCommand("DELETE FROM spoty WHERE nazev = @nazev AND popis = @popis AND adresa = @adresa AND mapa = @mapa", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@nazev", nazev);
                    _prikaz.Parameters.AddWithValue("@popis", popis);
                    _prikaz.Parameters.AddWithValue("@adresa", adresa);
                    _prikaz.Parameters.AddWithValue("@mapa", mapa);
                    _prikaz.ExecuteNonQuery();
                }
                sqlCe.UpdateSpoty(id, url, nazev, popis, praha, adresa, metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, obtiznost, typTreninku, zverejneni, typSpotu, idUzivatele, "NonID");
            }
            return id;
        }

        // vytvoří se nový obrázek v MySql databázi, uloží se jeho ID a smaže se a pomocí metody UpdateObrazky u SqlCe databáze se vytvoří i v SqlCe databázi

        public void VytvoreniObrazku(SqlCeDatabaze sqlCe, int idSpotu, string nahled, string obrazek, string typObrazku, int idUzivatele)
        {
            int id = -1;
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new MySqlCommand("INSERT INTO obrazky(id_spotu, nahled, obrazek) VALUES(@idSpotu, @nahled, @obrazek)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu);
                    _prikaz.Parameters.AddWithValue("@nahled", nahled);
                    _prikaz.Parameters.AddWithValue("@obrazek", obrazek);

                    _prikaz.ExecuteNonQuery();
                }
                using (_prikaz = new MySqlCommand("SELECT id FROM obrazky WHERE id_spotu = @idSpotu AND nahled = @nahled", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu);
                    _prikaz.Parameters.AddWithValue("@nahled", nahled);
                    using (_dataReader = _prikaz.ExecuteReader())
                        while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                }
                using (_prikaz = new MySqlCommand("DELETE FROM obrazky WHERE id_spotu = @idSpotu AND nahled = @nahled", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu);
                    _prikaz.Parameters.AddWithValue("@nahled", nahled);
                    _prikaz.ExecuteNonQuery();
                }
                sqlCe.UpdateObrazky(id, idSpotu, nahled, obrazek, typObrazku, idUzivatele);
            }
        }

        #endregion

        #region Oblíbené, hodnocené

        // přidá do tabulky oblibene uživatelův oblíbený spot, vrací ID tohoto oblíbeného
        public int PridaniSpotuDoOblibenych(int idSpotu, int idUzivatele)
        {
            int id = 0;
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new MySqlCommand("INSERT INTO oblibene(id_spotu, id_uzivatele, zverejneni) VALUES(@idSpotu, @idUzivatele, @zverejneni)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu);
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele);
                    _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Now);
                    _prikaz.ExecuteNonQuery();
                }

                using (_prikaz = new MySqlCommand("SELECT id FROM oblibene ORDER BY id DESC LIMIT 1", _pripojeni))
                {
                    using (_dataReader = _prikaz.ExecuteReader())
                        while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                }
            }
            return id;
        }

        // odebere uživatelovo oblíbené z databáze
        public void OdebraniSpotuZOblibenych(int idSpotu, int idUzivatele)
        {
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new MySqlCommand("DELETE FROM oblibene WHERE id_spotu = @idSpotu AND id_uzivatele = @idUzivatele", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu);
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele);
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        // přidá hodnocení do databáze
        public int OhodnotitSpot(int idSpotu, int hodnoceni, int idUzivatele)
        {
            int id = 0;
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new MySqlCommand("INSERT INTO hodnoceni(id_spotu, hodnoceni, id_uzivatele, zverejneni) VALUES(@idSpotu,@hodnoceni, @idUzivatele, @zverejneni)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu);
                    _prikaz.Parameters.AddWithValue("@hodnoceni", hodnoceni);
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele);
                    _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Now);
                    _prikaz.ExecuteNonQuery();
                }
                using (_prikaz = new MySqlCommand("SELECT id FROM hodnoceni ORDER BY id DESC LIMIT 1", _pripojeni))
                {
                    using (_dataReader = _prikaz.ExecuteReader())
                        while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                }
            }
            return id;
        }

        // načte hodnocení uživatele z databáze, vrátí hodnotu ohodnocení
        public string ZiskatHodnoceni(int idUzivatele, int idSpotu)
        {
            string hodnoceni = "";
            List<string> hodnoceniList = new List<string>();
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new MySqlCommand("SELECT id_spotu, hodnoceni, id_uzivatele FROM hodnoceni WHERE id_uzivatele = @idUzivatele AND id_spotu = @idSpotu", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele);
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu);
                    using (_dataReader = _prikaz.ExecuteReader()) while (_dataReader.Read()) hodnoceni = _dataReader["hodnoceni"].ToString();
                }
            }
            return hodnoceni;
        }

        #endregion

        #region Registrace, přihlášení

        // Zaregistruje uživatele, vloží ho do databáze
        public int RegistrovatSe(string uzivatel, string email, string heslo)
        {
            // zakóduji heslo
            heslo = heslo.HashPassword();
            var id = 0;
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new MySqlCommand("INSERT INTO uzivatele(uzivatel, url, email, heslo, obrazek, datum, stav, aktivace) VALUES(@uzivatel, '', @email, @heslo, 'default.png', @datum, 1, '123456789')", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@uzivatel", uzivatel);
                    _prikaz.Parameters.AddWithValue("@email", email);
                    _prikaz.Parameters.AddWithValue("@heslo", heslo);
                    _prikaz.Parameters.AddWithValue("@datum", DateTime.Now);
                    _prikaz.ExecuteNonQuery();
                }
                using (_prikaz = new MySqlCommand("SELECT id FROM uzivatele ORDER BY id DESC LIMIT 1", _pripojeni))
                {
                    using (_dataReader = _prikaz.ExecuteReader()) while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
                }
            }
            return id;
        }

        // Zjistí, jestli údaje o přihlášení jsou správné
        public void PrihlasitSe(string jmenoEmail, string heslo, ref bool prihlaseniOk, ref string uzivatel, ref int id)
        {
            // zakóduji heslo
            heslo = heslo.HashPassword();
            prihlaseniOk = false;
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new MySqlCommand("SELECT id, email, uzivatel FROM uzivatele WHERE (email = @email OR uzivatel = @jmeno) AND heslo = @heslo", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@jmeno", jmenoEmail);
                    _prikaz.Parameters.AddWithValue("@email", jmenoEmail);
                    _prikaz.Parameters.AddWithValue("@heslo", heslo);
                    using (_dataReader = _prikaz.ExecuteReader())
                        while (_dataReader.Read())
                        {
                            prihlaseniOk = true;
                            uzivatel = _dataReader["uzivatel"].ToString();
                            id = (int)_dataReader["id"];
                        }
                }
            }
        }

        #endregion

        #region Vybrání výsledků, Vytvoření příkazu SELECT, Načtení záznamů

        // Metoda definuje WHERE podmínku pro vyhledávání a vrací SELECT pomocí metody VytvoreniPrikazuVyhledavani()
        public string VybraniVysledku(string nazev, string castPrahy, string povrch, string obtiznost, string velikost, string typ)
        {
            nazev = (nazev == "Zadejte název spotu" || nazev == "") ? "" : "AND nazev like '%" + nazev + "%'";
            castPrahy = (castPrahy == "Vyberte" || castPrahy == "Nezáleží" || castPrahy == "") ? "" : "AND praha = '" + castPrahy + "'";
            povrch = (povrch == "Vyberte" || povrch == "Nezáleží" || povrch == "") ? "" : "AND povrch = '" + povrch + "'";
            obtiznost = (obtiznost == "Vyberte" || obtiznost == "Nezáleží" || obtiznost == "") ? "" : "AND obtiznost = '" + obtiznost + "'";
            velikost = (velikost == "Vyberte" || velikost == "Nezáleží" || velikost == "") ? "" : "AND velikost = '" + velikost + "'";
            typ = (typ == "Vyberte" || typ == "Nezáleží" || typ == "") ? "" : "AND typ_treninku = '" + typ + "'";
            string where = nazev + " " + castPrahy + " " + obtiznost + " " + velikost + " " + typ + " " + povrch + " ";
            where = " " + where.Trim() + " ";
            if (where.Trim() == "") { where = ""; }
            return VytvoreniPrikazuVyhledavani(where);
        }

        // Metoda definuje WHERE podmínku pro uživatelem ohodnocené spoty nebo uživatelovi oblíbené spoty a vrací SELECT pomocí metody VytvoreniPrikazuVyhledavani()
        public string VybraniVysledku(int idUzivatele, string uzivatelskaAkce)
        {
            string where = "";
            switch (uzivatelskaAkce)
            {
                case "hodnocení":
                    where = "AND h.id_uzivatele = " + idUzivatele + "";
                    break;
                case "oblíbené":
                    where = "AND o.id_uzivatele = " + idUzivatele + "";
                    break;
            }
            return VytvoreniPrikazuVyhledavani(where);
        }

        // Metoda resetuje WHERE podmínku a vrací SELECT pomocí metody VytvoreniPrikazuVyhledavani()
        public string VybraniVysledku()
        {
            string where = "";
            return VytvoreniPrikazuVyhledavani(where);
        }

        // Vytváří a vrací SELECT pro vyhledávání spotů
        private string VytvoreniPrikazuVyhledavani(string where)
        {
            string select = "SELECT s.id as 'ID', nazev as 'Název', praha AS 'ČástPrahy', povrch AS 'Povrch', obtiznost AS 'ObtížnostSkoků', velikost as 'Velikost', typ_treninku AS 'TypTréninku', TRUNCATE(COALESCE(AVG(h.hodnoceni), 0), 2) as 'Hodnocení' FROM spoty s LEFT OUTER JOIN hodnoceni h ON s.id = h.id_spotu LEFT OUTER JOIN oblibene o ON s.id = o.id_spotu WHERE 1=1 " + where + " GROUP BY ID, Název, 'ČástPrahy', Povrch, 'ObtížnostSkoků', velikost, 'TypTréninku' ORDER BY Název ASC";
            return select;
        }

        // Načítá data z databáze a podle zvoleného SELECT příkazu vrací dataSet pro překopírování do DataGridu
        public DataSet NacteniZaznamu(string select)
        {
            DataSet ds = new DataSet();
            MySqlDataAdapter mDataAdapter = new MySqlDataAdapter(select, _pripojeni);
            mDataAdapter.Fill(ds, "spoty"); //naplní data do DataSetu
            return ds;
        }

        #endregion
    }
}
