﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace DatabazeParkourovychSpotu
{
    /// <summary>
    /// Interaction logic for NovySpot.xaml
    /// </summary>
    public partial class NovySpot : Window
    {
        private MainWindow _mainWindow;
        private List<string> soubory = new List<string>();
        public NovySpot(MainWindow mw)
        {
            InitializeComponent();
            _mainWindow = mw;
        }

        // při kliknutí na tlačítko vytvořit spot
        // zkontroluje jestli jsou povinné položky vyplněné a poté se odešle spot
        private void vytvoreniSpotuButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = _mainWindow.ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                bool vytvoreniSpotuOk = true;
                TextBox nazev = nazevTextBox;
                TextBox popis = popisTextBox;
                ComboBox praha = prahaComboBox;
                ComboBox velikost = velikostComboBox;
                ComboBox povrch = povrchComboBox;
                ComboBox obtiznost = obtiznostComboBox;
                ComboBox typTreninku = typTreninkuComboBox;
                TextBox adresa = adresaTextBox;
                string url = "";
                TextBox metro = metroTextBox;
                TextBox bus = busTextBox;
                TextBox tramvaj = tramvajTextBox;
                TextBox mapa = mapaTextBox;
                TextBox streetview = streetviewTextBox;
                TextBox panorama = panoramaTextBox;
                DateTime zverejneni = DateTime.Now;
                string typZaznamu = "soukromý";
                int idUzivatele = _mainWindow.IdUzivatele;

                string adresaText = (adresa.Foreground.ToString() != "#FF606060") ? adresa.Text : "";
                string metroText = (metro.Foreground.ToString() != "#FF606060") ? metro.Text : "";
                string busText = (bus.Foreground.ToString() != "#FF606060") ? bus.Text : "";
                string tramvajText = (tramvaj.Foreground.ToString() != "#FF606060") ? tramvaj.Text : "";
                string mapaText = (mapa.Foreground.ToString() != "#FF606060") ? mapa.Text : "";
                string streetviewText = (streetview.Foreground.ToString() != "#FF606060") ? streetview.Text : "";
                string panoramaText = (panorama.Foreground.ToString() != "#FF606060") ? panorama.Text : "";

                if (nazev.Text.Length < 3 || nazev.Foreground.ToString() == "#FF606060")
                {
                    vytvoreniSpotuOk = false;
                    CustomMessageBox.Show("Nelze vytvořit spot, prosím vyplňte název spotu. Název musí mít alespoň 3 znaky.", "Přidání nového spotu");
                    nazev.Focus();
                }
                else
                if (popis.Text.Length < 20 || popis.Foreground.ToString() == "#FF606060")
                {
                    vytvoreniSpotuOk = false;
                    CustomMessageBox.Show("Nelze vytvořit spot, prosím vyplňte popis spotu. Popis musí mít alespoň 20 znaků.", "Přidání nového spotu");
                    popis.Focus();
                }
                else
                if (praha.Text == "Vyberte")
                {
                    vytvoreniSpotuOk = false;
                    CustomMessageBox.Show("Nelze vytvořit spot, prosím vyberte část Prahy, ve které se spot nachází.", "Přidání nového spotu");
                    praha.IsDropDownOpen = true;
                }
                else
                if (velikost.Text == "Vyberte")
                {
                    vytvoreniSpotuOk = false;
                    CustomMessageBox.Show("Nelze vytvořit spot, prosím vyberte velikost spotu.", "Přidání nového spotu");
                    velikost.IsDropDownOpen = true;
                }
                else
                if (povrch.Text == "Vyberte")
                {
                    vytvoreniSpotuOk = false;
                    CustomMessageBox.Show("Nelze vytvořit spot, prosím vyberte povrch, ze kterého je spot tvořený.", "Přidání nového spotu");
                    povrch.IsDropDownOpen = true;
                }
                else
                if (obtiznost.Text == "Vyberte")
                {
                    vytvoreniSpotuOk = false;
                    CustomMessageBox.Show("Nelze vytvořit spot, prosím vyberte obtížnost skoků na spotu.", "Přidání nového spotu");
                    obtiznost.IsDropDownOpen = true;
                }
                else
                if (typTreninku.Text == "Vyberte")
                {
                    vytvoreniSpotuOk = false;
                    CustomMessageBox.Show("Nelze vytvořit spot, prosím vyberte, co lze na spotu trénovat.", "Přidání nového spotu");
                    typTreninku.IsDropDownOpen = true;
                }
                else
                {
                    int spotIndex = -1;
                    spotIndex = (SqlCeDatabaze.Spoty).FindIndex(s => s.Nazev.ToLower() == nazev.Text.ToLower() && s.TypSpotu == "soukromý" && s.IdUzivatele == _mainWindow.IdUzivatele);
                    if (spotIndex > -1)
                    {
                        vytvoreniSpotuOk = false;
                        CustomMessageBox.Show("Nelze vytvořit spot, spot se stejným názvem již existuje. Zadejte prosím jiný název spotu.", "Přidání nového spotu");
                        nazev.Focus();
                    }
                }

                if (vytvoreniSpotuOk)
                {
                    if (soubory.Count() > 0)
                    {
                        foreach (string soubor in soubory)
                        {
                            string nazevSouboru = soubor.Substring(soubor.LastIndexOf(@"\") + 1);
                            nazevSouboru = nazevSouboru.Insert(nazevSouboru.LastIndexOf('.'), "_thumb"); ;
                            string baseDirectory = AppDomain.CurrentDomain.BaseDirectory;
                            string slozkaUrl = nazev.Text.CreateUrl();
                            string slozka = baseDirectory + "img\\spoty\\nahledy\\" + slozkaUrl;
                            Directory.CreateDirectory(@slozka);
                            if (!File.Exists(@slozka + "\\" + nazevSouboru)) File.Copy(soubor, @slozka + "\\" + nazevSouboru);
                        }
                    }
                    if (_mainWindow._mySqlPripojeno)
                    {
                        int idSpotu = _mainWindow._mySql.VytvoreniSpotu(_mainWindow._sqlCe, url, nazevTextBox.Text, popisTextBox.Text, prahaComboBox.Text, adresaText, metroText, busText, tramvajText, mapaText, streetviewText, panoramaText, velikostComboBox.Text, povrchComboBox.Text, obtiznostComboBox.Text, typTreninkuComboBox.Text, zverejneni, typZaznamu, idUzivatele);
                        if (soubory.Count() > 0)
                        {
                            foreach (string soubor in soubory)
                            {
                                string nazevSouboru = soubor.Substring(soubor.LastIndexOf(@"\") + 1);
                                nazevSouboru = nazevSouboru.Insert(nazevSouboru.LastIndexOf('.'), "_thumb"); ;
                                string slozkaUrl = nazev.Text.CreateUrl();
                                string nahled = slozkaUrl + "/" + nazevSouboru;
                                _mainWindow._mySql.VytvoreniObrazku(_mainWindow._sqlCe, idSpotu, nahled, "", typZaznamu, idUzivatele);
                            }
                        }
                    }
                    else
                    {
                        int idSpotu = _mainWindow._sqlCe.VytvoreniSpotu(url, nazevTextBox.Text, popisTextBox.Text, prahaComboBox.Text, adresaText, metroText, busText, tramvajText, mapaText, streetviewText, panoramaText, velikostComboBox.Text, povrchComboBox.Text, obtiznostComboBox.Text, typTreninkuComboBox.Text, zverejneni, typZaznamu, idUzivatele);
                        if (soubory.Count() > 0)
                        {
                            foreach (string soubor in soubory)
                            {
                                string nazevSouboru = soubor.Substring(soubor.LastIndexOf(@"\") + 1);
                                nazevSouboru = nazevSouboru.Insert(nazevSouboru.LastIndexOf('.'), "_thumb"); ;
                                string slozkaUrl = nazev.Text.CreateUrl();
                                string nahled = slozkaUrl + "/" + nazevSouboru;
                                _mainWindow._sqlCe.VytvoreniObrazku(_mainWindow._sqlCe, idSpotu, nahled, "", typZaznamu, idUzivatele);
                            }
                        }
                    }
                    _mainWindow._sqlCe.NacteniSpotu();
                    _mainWindow._sqlCe.NacteniObrazku();
                    if (_mainWindow.TypZobrazenychSpotu == "Moje vlastní spoty") _mainWindow.NaplneniDataGriduVlastniSpoty();
                    CustomMessageBox.Show("Spot je úspěšně vytvořený. Detail spotu si lze prohlédnout po rozkliknutí Vašich spotů na hlavní stránce.", "Přidání nového spotu.");

                    foreach (Visual v in hlavniGrid.GetChildren())
                    {
                        if (v is TextBox)
                        {
                            TextBox t = (TextBox)v;
                            if (t.Text != "" && t.Name != "PART_EditableTextBox") t.InitialValue();
                        }
                        if (v is ComboBox)
                        {
                            ComboBox c = (ComboBox)v;
                            c.Text = "Vyberte";
                        }
                    }
                    if (soubory.Count > 0) soubory.Clear();
                }
            }
            else
            {
                CustomMessageBox.Show("Proběhla změna internetového připojení. Pro přidání spotu odešlete spot ještě jednou kliknutím na tlačítko 'Vytvořit spot'. Děkujeme.");
            }
        }

        // po kliknutí na tlčítko vybrání fotek
        private void pridatFotkyButton_Click(object sender, RoutedEventArgs e)
        {
            Button b = (Button)sender;
            if (b.Content.ToString() == "Přidat fotky")
            {
                // Vytvoření dialogu
                Microsoft.Win32.OpenFileDialog dialog = new Microsoft.Win32.OpenFileDialog();

                // Filtry a nastavení dialogu
                dialog.DefaultExt = ".jpg";
                dialog.Filter = "Obrázky (*.jpg, *.jpeg, *.png) | *.jpg; *.jpeg; *.png";
                dialog.Multiselect = true;
                dialog.Title = "Výběr fotek";

                // Zobrazení dialogu a uložení do proměnné
                Nullable<bool> result = dialog.ShowDialog();

                // Když vyberu soubory
                if (result == true)
                {
                    // vyberu cestu fotek, zobrazím
                    string filename = dialog.FileName;
                    int pocetFotek = dialog.FileNames.Count();
                    if (pocetFotek > 1) fotkyTextBox.Text = "Počet vybraných fotek: " + pocetFotek;
                    else fotkyTextBox.Text = filename.Substring(filename.LastIndexOf(@"\") + 1);

                    // přidám do listu
                    foreach (string soubor in dialog.FileNames) soubory.Add(soubor);
                    b.Content = "Smazat vybrané fotky";
                }
            }
            else
            {
                // smažu fotky
                fotkyTextBox.InitialValue();
                if (soubory.Count > 0) soubory.Clear();
                b.Content = "Přidat fotky";
            }
        }

        // volají se metody po přidání / odebrání focusu na textboxy
        private void TextBox_GotFocus(object sender, RoutedEventArgs e)
        {
            ((TextBox)sender).OnGotFocus();
        }

        private void TextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            ((TextBox)sender).OnLostFocus();
        }

        // nastavuje chování prvků při stisknutí enteru
        private async void NovySpotWindow_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (
                nazevTextBox.IsFocused ||
                popisTextBox.IsFocused ||
                prahaComboBox.IsFocused ||
                adresaTextBox.IsFocused ||
                metroTextBox.IsFocused ||
                busTextBox.IsFocused ||
                tramvajTextBox.IsFocused ||
                mapaTextBox.IsFocused ||
                streetviewTextBox.IsFocused ||
                panoramaTextBox.IsFocused ||
                velikostComboBox.IsFocused ||
                povrchComboBox.IsFocused ||
                obtiznostComboBox.IsFocused ||
                typTreninkuComboBox.IsFocused
                )
            {
                if ((e.Key.Equals(Key.Enter)) || (e.Key.Equals(Key.Return)))
                {
                    vytvoreniSpotuButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                    vytvoreniSpotuButton.Background = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));
                    vytvoreniSpotuButton.BorderBrush = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));

                    await Task.Delay(200);

                    vytvoreniSpotuButton.ClearValue(Button.BackgroundProperty);
                    vytvoreniSpotuButton.ClearValue(Button.BorderBrushProperty);
                    return;
                }
            }
            if (e.Key.Equals(Key.Escape))
            {
                this.Close();
                return;
            }
            foreach (Visual v in hlavniGrid.GetChildren())
            {
                if (v is Button)
                {
                    Button button = (Button)v;
                    if (((e.Key.Equals(Key.Enter)) || (e.Key.Equals(Key.Return))) && button.IsFocused)
                    {
                        button.Background = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));
                        button.BorderBrush = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));

                        await Task.Delay(200);

                        button.ClearValue(Button.BackgroundProperty);
                        button.ClearValue(Button.BorderBrushProperty);
                    }
                }
            }
        }
    }
}
